<?php

/**
 * @file
 * Contains \Drupal\ip2country\Form\IP2CountrySettingsForm.
 */

namespace Drupal\ip2country\Form;

use Drupal\Core\Form\ConfigFormBase;


/** Utility functions for loading IP/Country DB from external sources */
module_load_include('inc', 'ip2country');


/**
 * Configure ip2country settings for this site.
 */
class IP2CountrySettingsForm extends ConfigFormBase {

  /**
   * Implements \Drupal\Core\Form\FormInterface::getFormID().
   */
  public function getFormID() {
    return 'ip2country_admin_settings';
  }

  /**
   * Implements \Drupal\Core\Form\FormInterface::buildForm().
   */
  public function buildForm(array $form, array &$form_state) {
    $ip2country_config = $this->configFactory->get('ip2country.settings');

    drupal_add_js(drupal_get_path('module', 'ip2country') . '/ip2country.js');
    drupal_add_css(drupal_get_path('module', 'ip2country') . '/ip2country.css');

    // Container for database update preference forms.
    $form['ip2country_database_update'] = array(
      '#type'        => 'fieldset',
      '#title'       => t('Database updates'),
      '#collapsible' => FALSE,
      '#collapsed'   => FALSE,
    );

    // Form to enable watchdog logging of updates.
    $form['ip2country_database_update']['ip2country_watchdog'] = array(
      '#type'          => 'checkbox',
      '#title'         => t('Log database updates to watchdog'),
      '#default_value' => $ip2country_config->get('watchdog'),
    );

    // Form to choose RIR.
    $form['ip2country_database_update']['ip2country_rir'] = array(
      '#type'          => 'select',
      '#title'         => t('Regional Internet Registry'),
      '#options'       => array('afrinic' => 'AFRINIC', 'apnic' => 'APNIC', 'arin' => 'ARIN', 'lacnic' => 'LACNIC', 'ripe' => 'RIPE'),
      '#default_value' => $ip2country_config->get('rir'),
      '#description'   => t('Database will be downloaded from the selected RIR. You may find that the regional server nearest you has the best response time, but note that AFRINIC provides only its own subset of registration data.'),
    );

    // Form to enable MD5 checksum of downloaded databases.
    $form['ip2country_database_update']['ip2country_md5_checksum'] = array(
      '#type'          => 'checkbox',
      '#title'         => t('Perform MD5 checksum comparison'),
      '#description'   => t("Compare MD5 checksum downloaded from the RIR with MD5 checksum calculated locally to ensure the data has not been corrupted. RIRs don't always store current checksums, so if this option is checked your database updates may sometimes fail."),
      '#default_value' => $ip2country_config->get('md5_checksum'),
    );


    $period = drupal_map_assoc(array(86400, 302400, 604800, 1209600, 2419200), 'format_interval');
    $period[0] = t('Never');

    // Form to set automatic update interval.
    $form['ip2country_database_update']['ip2country_update_interval'] = array(
      '#type'          => 'select',
      '#title'         => t('Database update frequency'),
      '#default_value' => $ip2country_config->get('update_interval'),
      '#options'       => $period,
      '#description'   => t('Database will be automatically updated via cron.php. Cron must be enabled for this to work. Default period is 1 week (604800 seconds).'),
    );

    $update_time = \Drupal::state()->get('ip2country_last_update');
    if (!empty($update_time)) {
      $message = t('Database last updated on @date at @time from @registry server.', array('@date' => format_date($update_time, 'custom', 'n/j/Y'), '@time' => format_date($update_time, 'custom', 'H:i:s T'), '@registry' => drupal_strtoupper(\Drupal::state()->get('ip2country_last_update_rir'))));
    }
    else {
      $message = t('Database is empty. You may fill the database by pressing the @update button.', array('@update' => t('Update')));
    }

    // Form to initiate manual updating of the IP-Country database.
    $form['ip2country_database_update']['ip2country_update_database'] = array(
      '#type'        => 'button',
      '#value'       => t('Update'),
      '#executes_submit_callback' => FALSE,
      '#prefix'      => '<p>' . t('The IP to Country Database may be updated manually by pressing the "Update" button below. Note, this may take several minutes. Changes to the above settings will not be permanently saved unless you press the "Save configuration" button at the bottom of this page.') . '</p>',
      '#suffix'      => '<span id="dbthrobber" class="message">' . $message . '</span>',
    );

    // Container for manual lookup.
    $form['ip2country_manual_lookup'] = array(
      '#type'        => 'fieldset',
      '#title'       => t('Manual lookup'),
      '#description' => t('Examine database values'),
      '#collapsible' => FALSE,
      '#collapsed'   => FALSE,
    );

    // Form for IP address for manual lookup.
    $form['ip2country_manual_lookup']['ip2country_lookup'] = array(
      '#type'        => 'textfield',
      '#title'       => t('Manual lookup'),
      '#description' => t('Enter IP address'),
    );

    // Form to initiate manual lookup.
    $form['ip2country_manual_lookup']['ip2country_lookup_button'] = array(
      '#type'        => 'button',
      '#value'       => t('Lookup'),
      '#executes_submit_callback' => FALSE,
      '#prefix'      => '<div>' . t('An IP address may be looked up in the database by entering the address above then pressing the Lookup button below.') . '</div>',
      '#suffix'      => '<span id="lookup-message" class="message"></span>',
    );

    // Container for debugging preference forms.
    $form['ip2country_debug_preferences'] = array(
      '#type'        => 'fieldset',
      '#title'       => t('Debug preferences'),
      '#description' => t('Set debugging values'),
      '#collapsible' => FALSE,
      '#collapsed'   => FALSE,
    );

    // Form to turn on debugging.
    $form['ip2country_debug_preferences']['ip2country_debug'] = array(
      '#type'          => 'checkbox',
      '#title'         => t('Admin debug'),
      '#default_value' => $ip2country_config->get('debug'),
      '#description'   => t('Enables administrator to spoof an IP Address or Country for debugging purposes.'),
    );

    // Form to select Dummy Country or Dummy IP Address for testing.
    $form['ip2country_debug_preferences']['ip2country_test_type'] = array(
      '#type'          => 'radios',
      '#title'         => t('Select which parameter to spoof'),
      '#default_value' => $ip2country_config->get('test_type'),
      '#options'       => array(t('Country'), t('IP Address')),
    );

    //$ip_current = ip2country_get_country(\Drupal::request()->getClientIp());
    //$default_country = isset($ip2country_config->get('test_country')) ?
    //                   $ip2country_config->get('test_country')        :
    //                   $ip_current;
    $default_country = $ip2country_config->get('test_country');

    // Form to enter Country to spoof.
    $form['ip2country_debug_preferences']['ip2country_test_country'] = array(
      '#type'          => 'select',
      '#title'         => t('Country to use for testing'),
      '#default_value' => $default_country,
      '#options'       => \Drupal::service('country_manager')->getList(),
    );

    // Form to enter IP address to spoof.
    $form['ip2country_debug_preferences']['ip2country_test_ip_address'] = array(
      '#type'          => 'textfield',
      '#title'         => t('IP address to use for testing'),
      '#default_value' => $ip2country_config->get('test_ip_address', \Drupal::request()->getClientIp()),
    );

    return parent::buildForm($form, $form_state);
  }

  /**
   * Implements \Drupal\Core\Form\FormInterface::validateForm().
   */
  public function validateForm(array &$form, array &$form_state) {
    return parent::validateForm($form, $form_state);
  }

  /**
   * Implements \Drupal\Core\Form\FormInterface::submitForm().
   */
  public function submitForm(array &$form, array &$form_state) {
    $values = $form_state['values'];

    $ip2country_config = $this->configFactory->get('ip2country.settings');

    $ip2country_config
      ->setData(array(
        'watchdog' => $values['ip2country_watchdog'],
        'rir' => $values['ip2country_rir'],
        'md5_checksum' => $values['ip2country_md5_checksum'],
        'update_interval' => $values['ip2country_update_interval'],
        'debug' => $values['ip2country_debug'],
        'test_type' => $values['ip2country_test_type'],
        'test_country' => $values['ip2country_test_country'],
        'test_ip_address' => $values['ip2country_test_ip_address'],
      ))
      ->save();

    // Check to see if debug set.
    if ($values['ip2country_debug']) {
      // Debug on.
      if ($values['ip2country_test_type']) {
        // Dummy IP Address.
        $ip = $values['ip2country_test_ip_address'];
        $country_code = ip2country_get_country($ip);
      }
      else {
        // Dummy Country.
        $country_code = $values['ip2country_test_country'];
      }
      $country_list = \Drupal::service('country_manager')->getList();
      $country_name = $country_list[$country_code];
      drupal_set_message(t('Using DEBUG value for Country - @country (@code)', array('@country' => $country_name, '@code'    => $country_code)));
    }
    else {
      // Debug off - make sure we set/reset IP/Country to their real values.
      $ip = \Drupal::request()->getClientIp();
      $country_code = ip2country_get_country($ip);
      drupal_set_message(t('Using ACTUAL value for Country - @country', array('@country' => $country_code)));
    }

    // Finally, save country, if it has been determined.
    if ($country_code) {
      // Store the ISO country code in the $user object.
      $account_id = \Drupal::currentUser()->id();
      $account = user_load($account_id);
      $account->country_iso_code_2 = $country_code;
      \Drupal::service('user.data')->set('ip2country', $account_id, 'country_iso_code_2', $country_code);
    }

    parent::submitForm($form, $form_state);
  }

}
